package com.qmth.cet.plug.service.impl;

import com.qmth.cet.plug.service.ExeInvokerService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.concurrent.TimeUnit;

@Service
public class ExeInvokerServiceImpl implements ExeInvokerService {

    private static final Logger logger = LoggerFactory.getLogger(ExeInvokerServiceImpl.class);

    @Value("${com.qmth.client.dir}")
    private String exePath;

    /**
     * @param courseCode     课程代码
     * @param inputFilePath  待处理的文件路径
     * @param outputFilePath 处理后的文件路径
     */
    @Override
    public boolean invokeExternalExe(String courseCode, String inputFilePath, String outputFilePath) {
        // 参数校验
        if (courseCode == null || inputFilePath == null || outputFilePath == null) {
            logger.error("参数不能为空");
            return false;
        }

        if (exePath == null || exePath.trim().isEmpty()) {
            logger.error("EXE路径未配置");
            return false;
        }

        String[] commands = {exePath + File.separator + "omr.exe", courseCode, inputFilePath, outputFilePath};

        Process process = null;
        try {
            ProcessBuilder processBuilder = new ProcessBuilder(commands);
            processBuilder.directory(new File(exePath));
            processBuilder.redirectErrorStream(true);
            process = processBuilder.start();

            StringBuilder output = new StringBuilder();
            try (BufferedReader reader = new BufferedReader(
                    new InputStreamReader(process.getInputStream()))) {
                String line;
                while ((line = reader.readLine()) != null) {
                    output.append(line).append(System.lineSeparator());
                }
            }

            // 设置超时时间，防止无限等待
            boolean exited = process.waitFor(10, TimeUnit.MINUTES);
            if (!exited) {
                process.destroyForcibly();
                logger.error("执行超时，已强制终止");
                return false;
            }

            int exitCode = process.exitValue();
            if (exitCode == 0) {
                logger.info("执行成功！");
                return true;
            } else {
                logger.error("执行失败！退出码：{}，错误信息：{}", exitCode, output);
                return false;
            }
        } catch (IOException | InterruptedException e) {
            Thread.currentThread().interrupt(); // 重置中断状态
            logger.error("调用EXE失败：{}", e.getMessage());
            return false;
        } finally {
            // 显式关闭所有流
            if (process != null) {
                process.destroy();
            }
        }
    }

}
